require("vec2")

function love.load()

	if arg[#arg] == "-debug"
	then
		require("mobdebug").start()
	else
		debug = nil
	end

  love.physics.setMeter(64) --the height of a meter our worlds will be 64px
  world = love.physics.newWorld(0, 9.81*64, true) --create a world for the bodies to exist in with horizontal gravity of 0 and vertical gravity of 9.81
 
  objects = {} -- table to hold all our physical objects
 
  --let's create the ground
  objects.ground = {}
  objects.ground.body = love.physics.newBody(world, 650/2, 650-50/2) --remember, the shape (the rectangle we create next) anchors to the body from its center, so we have to move it to (650/2, 650-50/2)
  objects.ground.shape = love.physics.newRectangleShape(650, 50) --make a rectangle with a width of 650 and a height of 50
  objects.ground.fixture = love.physics.newFixture(objects.ground.body, objects.ground.shape); --attach shape to body
 
  --let's create a ball
  objects.ball = {}
  objects.ball.body = love.physics.newBody(world, 650/2, 650/2, "dynamic") --place the body in the center of the world and make it dynamic, so it can move around
  objects.ball.shape = love.physics.newCircleShape(20) --the ball's shape has a radius of 20
  objects.ball.fixture = love.physics.newFixture(objects.ball.body, objects.ball.shape, 1) -- Attach fixture to body and give it a density of 1.
  objects.ball.fixture:setRestitution(0.5) --let the ball bounce
  objects.ball.body:setLinearDamping(1)
  objects.ball.angle = 0
 
  --let's create a couple blocks to play around with
  objects.block1 = {}
  objects.block1.body = love.physics.newBody(world, 200, 550, "dynamic")
  objects.block1.shape = love.physics.newRectangleShape(0, 0, 50, 100)
  objects.block1.fixture = love.physics.newFixture(objects.block1.body, objects.block1.shape, 5) -- A higher density gives it more mass.
 
  objects.block2 = {}
  objects.block2.body = love.physics.newBody(world, 200, 400, "dynamic")
  objects.block2.shape = love.physics.newRectangleShape(0, 0, 100, 50)
  objects.block2.fixture = love.physics.newFixture(objects.block2.body, objects.block2.shape, 2)
 
  --initial graphics setup
  love.graphics.setBackgroundColor(0.41, 0.53, 0.97) --set the background color to a nice blue
  love.window.setMode(650, 650) --set the window dimensions to 650 by 650
  
  Font = love.graphics.newFont("UbuntuMono-R.ttf")
  objects.text = love.graphics.newText(Font, "Init")
end
 
 
function love.update(dt)

  world:update(dt) --this puts the world into motion
 
	Force = vec2(0,0)
	Speed = vec2(objects.ball.body:getLinearVelocity())
	Angle = objects.ball.body:getAngle()
 
  -- here we are going to create some keyboard events
  if love.keyboard.isDown("right") then --press the right arrow key to push the ball to the right
    Force.x = Force.x + 200
  end
  if love.keyboard.isDown("left") then --press the left arrow key to push the ball to the left
    Force.x = Force.x - 200
  end
  -- if love.keyboard.isDown("up") then --press the up arrow key to set the ball in the air
	-- objects.ball.body:setY(100)
    -- objects.ball.body:applyForce(0, -500)   -- -objects.ball.body:getMass() * 9.81 * 100)
    -- objects.ball.body:setLinearVelocity(0, 0) --we must set the velocity to zero to prevent a potentially large velocity generated by the change in position
  -- end
  
   -- objects.ball.angle = math.atan((vy/vx), (vx/vy))
   -- if vx < 0 then 											-- fix, da die atan-Funktion negative Werte für cos nicht vearabeitet (?)
	 -- objects.ball.angle = objects.ball.angle - math.pi
   -- end
  
  Force.y = -0.75 * math.abs(Speed.x)   -- -objects.ball.body:getMass() * 9.81 * 100)

  print ("main.lua:81  Angle = " .. Angle) 
  Force.rotate(Angle)
  
  objects.ball.body:applyForce(Force.x, Force.y)

  -- wrap the ball on the left and the right
  if objects.ball.body:getX() < 0 then
	objects.ball.body:setX(love.graphics.getWidth())
  elseif objects.ball.body:getX() > love.graphics.getWidth() then
	objects.ball.body:setX(0)
  end

  objects.text:set("Angle = " .. objects.ball.angle)
  
end
 
function love.draw()
  love.graphics.setColor(0.28, 0.63, 0.05) -- set the drawing color to green for the ground
  love.graphics.polygon("fill", objects.ground.body:getWorldPoints(objects.ground.shape:getPoints())) -- draw a "filled in" polygon using the ground's coordinates
 
  local radius = objects.ball.shape:getRadius()
  local X = objects.ball.body:getX()
  local Y = objects.ball.body:getY()
  love.graphics.setColor(0.76, 0.18, 0.05) --set the drawing color to red for the ball
  love.graphics.circle("fill", X, Y, radius)
  love.graphics.setColor(0, 0, 0) 
   
  love.graphics.line(X, Y, X + radius * math.cos (objects.ball.angle), Y + radius * math.sin (objects.ball.angle))    -- Angle-Anzeiger
  
  -- Kraftvektor-Anzeiger
 	
 
  love.graphics.setColor(0.20, 0.20, 0.20) -- set the drawing color to grey for the blocks
  love.graphics.polygon("fill", objects.block1.body:getWorldPoints(objects.block1.shape:getPoints()))
  love.graphics.polygon("fill", objects.block2.body:getWorldPoints(objects.block2.shape:getPoints()))
  
  love.graphics.setColor(1,1,1)
  love.graphics.draw(objects.text, 10,10)
end




function love.keypressed(key, unicode)
if key == 'escape'
	then
		love.event.push('quit')
	end
end